<?php

declare(strict_types=1);

namespace Apexl\Tests\Core\FeatureTests\Relations;

use apexl\Io\TestInfrastructure\FeatureTest;
use apexl\Io\TestInfrastructure\SpoofsGlobalUser;
use DI\DependencyException;
use DI\NotFoundException;

abstract class AbstractRelationTest extends FeatureTest
{
    use SpoofsGlobalUser;

    /**
     * @throws DependencyException
     * @throws NotFoundException
     */
    protected function setUp(): void
    {
        $this->addGlobalUser();
    }
}