<?php

namespace apexl\Io\modules\installDisplay;

use apexl\Io\includes\Controller;
use apexl\Io\services\Output;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

final readonly class installDisplayController extends Controller
{
    public function __invoke(Output $output, ServerRequestInterface $request, ResponseInterface $response): ResponseInterface
    {
        //We're installing
        $output->addResponse($request, ['thing' => 'stuff']);

        return $this->json($response, []);
    }
}
