<?php

declare(strict_types=1);

namespace apexl\Io\factories;

use apexl\Io\services\Mailer;
use DI\Container;

final readonly class MailerFactory
{
    public function __construct(private Container $container) {}

    public function make(): Mailer
    {
        return $this->container->make(Mailer::class);
    }
}