<?php

declare(strict_types=1);

namespace apexl\Io\interfaces;

use apexl\Io\dto\Relationship\HasOneViaPivot;

interface HasNotesInterface
{
    public function latestNoteRelation(): HasOneViaPivot;

}